const system = require("../../system");
const uuidv4 = require('uuid/v4');
const md5 = require("MD5");
var settings = require("../../../config/settings");

class OpPlatformUtils {
    constructor() {
        this.restClient = system.getObject("util.restClient");
        this.createUserUrl = settings.paasUrl() + "api/auth/accessAuth/register";
        this.fetchDefaultVCodeUrl = settings.paasUrl() + "api/auth/accessAuth/fetchDefaultVCode";
        this.loginUrl = settings.paasUrl() + "api/auth/accessAuth/login";
        this.loginByVCodeUrl = settings.paasUrl() + "api/auth/accessAuth/loginByVCode";
        this.modiPasswordByMobileUrl = settings.paasUrl() + "api/auth/accessAuth/modiPasswordByMobile";
    }
    getUUID() {
        var uuid = uuidv4();
        var u = uuid.replace(/\-/g, "");
        return u;
    }
    async getReqApiAccessKey(appKey, secret) {
        var cacheManager = system.getObject("db.common.cacheManager");
        var reqApiAccessKey = null;
        if (appKey && secret) {
            reqApiAccessKey = await cacheManager["ApiAccessKeyCache"].cache(appKey, null, system.exTime, secret);
        } else {
            reqApiAccessKey = await cacheManager["ApiAccessKeyCache"].cache(settings.appKey, null, system.exTime);
        }
        if (!reqApiAccessKey || !reqApiAccessKey.data) {
            return system.getResult(null, "获取请求token失败");
        }
        return reqApiAccessKey;
    }
    /**
     * 创建用户信息
     * @param {*} userName 用户名
     * @param {*} mobile 手机号
     * @param {*} password 密码，不传为使用默认密码
     * 
     * 返回值：
     * {
            "status": 0,---值为2000为已经存在此用户，注册失败
            "msg": "success",
            "data": {
                "auth_url": "http://sj.app.com:3002/auth?opencode=1e4949d1c39444a8b32f023143625b1d",---回调url，通过回调地址获取平台用户信息
                "opencode": "1e4949d1c39444a8b32f023143625b1d",---平台用户code随机生成会变，平台是30s有效期，通过其可以向获取用户信息
                "open_user_id": 12---平台用户id
            },
            "requestid": "5362bf6f941e4f92961a61068f05cd7f"
        }
     */
    async createUserInfo(userName, mobile, password, appKey, secret) {
        var reqApiAccessKey = await this.getReqApiAccessKey(appKey, secret);
        if (reqApiAccessKey.status != 0) {
            return reqApiAccessKey;
        }
        var param = {
            userName: userName,
            mobile: mobile,
            password: password || settings.defaultPassWord,

        }
        //按照访问token
        var restResult = await this.restClient.execPostWithAK(
            param,
            this.createUserUrl, reqApiAccessKey.data.accessKey);

        if (restResult.status != 0 || !restResult.data) {
            return system.getResult(restResult.status, restResult.msg);
        }
        return system.getResultSuccess(restResult.data);
    }
    /**
     * 
     * @param {*} mobile 手机号
     */
    async fetchVCode(mobile, appKey, secret) {
        var reqApiAccessKey = await this.getReqApiAccessKey(appKey, secret);
        if (reqApiAccessKey.status != 0) {
            return reqApiAccessKey;
        }
        var param = { mobile: mobile }
        //按照访问token
        var restResult = await this.restClient.execPostWithAK(
            param,
            this.fetchDefaultVCodeUrl, reqApiAccessKey.data.accessKey);

        if (restResult.status != 0 || !restResult.data) {
            return system.getResult(null, restResult.msg);
        }
        return system.getResultSuccess();
    }
    /**
        * 创建用户信息
        * @param {*} userName 用户名
        * @param {*} mobile 手机号
        * @param {*} password 密码，不传为使用默认密码
        * 
        * 返回值：
        * {
               "status": 0,---值为2000为已经存在此用户，注册失败
               "msg": "success",
               "data": {
                   "auth_url": "http://sj.app.com:3002/auth?opencode=1e4949d1c39444a8b32f023143625b1d",---回调url，通过回调地址获取平台用户信息
                   "opencode": "1e4949d1c39444a8b32f023143625b1d",---平台用户code随机生成会变，平台是30s有效期，通过其可以向获取用户信息
                   "open_user_id": 12---平台用户id
               },
               "requestid": "5362bf6f941e4f92961a61068f05cd7f"
           }
        */
    async loginByVCode(mobile, vcode, password, appKey, secret) {
        var reqApiAccessKey = await this.getReqApiAccessKey(appKey, secret);
        if (reqApiAccessKey.status != 0) {
            return reqApiAccessKey;
        }
        var param = {
            mobile: mobile,
            vcode: vcode,
            password: password || ""
        }
        //按照访问token
        var restResult = await this.restClient.execPostWithAK(
            param,
            this.loginByVCodeUrl, reqApiAccessKey.data.accessKey);

        if (restResult.status != 0 || !restResult.data) {
            return system.getResult(restResult.status, restResult.msg);
        }
        return system.getResultSuccess(restResult.data);
    }
    /**
     * 用户登录
     * @param {*} userName 用户名
     * @param {*} password 密码，不传为使用默认密码
     * 
     * 返回值：
     * {
            "status": 0,---值为2010为用户名或密码错误
            "msg": "success",
            "data": {
                "auth_url": "http://sj.app.com:3002/auth?opencode=1e4949d1c39444a8b32f023143625b1d",---回调url，通过回调地址获取平台用户信息
                "opencode": "1e4949d1c39444a8b32f023143625b1d"---平台用户code随机生成会变，平台是30s有效期，通过其可以向获取用户信息
            },
            "requestid": "5362bf6f941e4f92961a61068f05cd7f"
        }
     */
    async login(userName, password, appKey, secret) {
        var reqApiAccessKey = await this.getReqApiAccessKey(appKey, secret);
        if (reqApiAccessKey.status != 0) {
            return reqApiAccessKey;
        }
        var param = {
            userName: userName,
            password: password || settings.defaultPassWord,

        }
        //按照访问token
        var restResult = await this.restClient.execPostWithAK(
            param,
            this.loginUrl, reqApiAccessKey.data.accessKey);

        if (restResult.status != 0 || !restResult.data) {
            return system.getResultFail(restResult.status, restResult.msg);
        }
        return system.getResultSuccess(restResult.data);
    }
    /**
     * 通过opencode获取用户登录信息
     * @param {*} opencode 用户登录或注册opencode
     * 
     * 返回值：
     * {
            "status": 0,---值为2010为用户名或密码错误
            "msg": "success",
            "data": {},---平台用户信息
            "requestid": "5362bf6f941e4f92961a61068f05cd7f"
        }
    */
    async authByCode(opencode, appKey, secret) {
        var reqApiAccessKey = await this.getReqApiAccessKey(appKey, secret);
        if (reqApiAccessKey.status != 0) {
            return reqApiAccessKey;
        }
        var param = {
            opencode: opencode
        }
        //按照访问token
        var restResult = await this.restClient.execPostWithAK(
            param,
            this.authByCodeUrl, reqApiAccessKey.data.accessKey);

        if (restResult.status != 0 || !restResult.data) {
            return system.getResult(restResult.status, restResult.msg);
        }
        return system.getResultSuccess(restResult.data);
    }

    async modiPasswordByMobile(mobile, vcode, newPwd, appKey, secret) {
        var reqApiAccessKey = await this.getReqApiAccessKey(appKey, secret);
        if (reqApiAccessKey.status != 0) {
            return reqApiAccessKey;
        }
        var param = { mobile: mobile, vcode: vcode, newPwd: newPwd }
        //按照访问token
        var restResult = await this.restClient.execPostWithAK(
            param,
            this.modiPasswordByMobileUrl, reqApiAccessKey.data.accessKey);

        if (restResult.status != 0 || !restResult.data) {
            return system.getResult(null, restResult.msg);
        }
        return system.getResultSuccess();
    }

    //------------------------新的方式------------------------------------------------------------------------------------
    async getReqTokenByHosts(app_hosts) {
        var cacheManager = system.getObject("db.common.cacheManager");
        var result = await cacheManager["AppTokenByHostsCache"].cache(app_hosts, null, system.shortExTime);
        return result;
    }
    async getReqUserPinByChannelUserId(pobj) {
        var inputkey = pobj.appInfo.uapp_key + "_" + pobj.actionBody.channelUserId;
        var cacheManager = system.getObject("db.common.cacheManager");
        var result = await cacheManager["AppUserPinByChannelUserIdCache"].getCache(inputkey, pobj, system.shortExTime);
        if (result && result.status == 0) {
            return system.getResultFail(system.reDoLoginFail, "请勿重复登录");
        }
        result = await cacheManager["AppUserPinByChannelUserIdCache"].cache(inputkey, pobj, system.shortExTime);
        return result;
    }
    async getReqUserPinByLgoin(pobj) {
        var inputkey = pobj.appInfo.uapp_key + "_" + pobj.actionBody.userName;
        var cacheManager = system.getObject("db.common.cacheManager");
        var result = await cacheManager["AppUserPinByLoginPwdCache"].getCache(inputkey, pobj, system.shortExTime);
        if (result && result.status == 0) {
            return system.getResultFail(system.reDoLoginFail, "请勿重复登录");
        }
        result = await cacheManager["AppUserPinByLoginPwdCache"].cache(inputkey, pobj, system.shortExTime);
        return result;
    }
    async getReqUserPinByLgoinVcode(pobj) {
        var inputkey = pobj.appInfo.uapp_key + "_" + pobj.actionBody.mobile;
        var cacheManager = system.getObject("db.common.cacheManager");
        var result = await cacheManager["AppUserPinByLoginVcodeCache"].getCache(inputkey, pobj, system.shortExTime);
        if (result && result.status == 0) {
            return system.getResultFail(system.reDoLoginFail, "请勿重复登录");
        }
        result = await cacheManager["AppUserPinByLoginVcodeCache"].cache(inputkey, pobj, system.shortExTime);
        return result;
    }

    async putUserPwdByMobile(pobj, actionBody) {
        if (!actionBody.mobile) {
            return system.getResult(null, "pobj.mobile can not be empty ！");
        }
        if (!actionBody.vcode) {
            return system.getResult(null, "pobj.vcode can not be empty ！");
        }
        if (!actionBody.newPwd) {
            return system.getResult(null, "pobj.newPwd can not be empty ！");
        }
        if (!actionBody.appInfo) {
            return system.getResult(null, "pobj.appInfo can not be empty ！");
        }
        var acckapp = await this.modiPasswordByMobile(actionBody.mobile, actionBody.vcode, actionBody.newPwd, pobj.appInfo.uapp_key, pobj.appInfo.uapp_secret);
        return acckapp;
    }

}

module.exports = OpPlatformUtils;
